@@ -20,6 +20,7 @@
  */
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <telepathy-glib/channel.h>
 #include <telepathy-glib/connection.h>
@@ -93,6 +94,8 @@ folks_tp_lowlevel_connection_open_contact_list_channel_async (
       folks_tp_lowlevel_connection_open_contact_list_channel_finish);
   tp_cli_connection_interface_requests_call_ensure_channel (conn, -1, request,
       connection_ensure_channel_cb, result, NULL, G_OBJECT (conn));
+
+  g_hash_table_unref (request);
 }
 
 /**
@@ -130,6 +133,80 @@ folks_tp_lowlevel_connection_open_contact_list_channel_finish (
       G_SIMPLE_ASYNC_RESULT (result));
 }
 
+static void
+connection_get_alias_flags_cb (TpConnection *conn,
+    guint flags,
+    const GError *error,
+    gpointer user_data,
+    GObject *weak_object)
+{
+  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (user_data);
+
+  if (error != NULL)
+    {
+      g_simple_async_result_set_from_error (simple, error);
+    }
+  else
+    {
+      g_simple_async_result_set_op_res_gpointer (simple,
+          GUINT_TO_POINTER (flags), NULL);
+    }
+
+  g_simple_async_result_complete (simple);
+  g_object_unref (simple);
+}
+
+void
+folks_tp_lowlevel_connection_get_alias_flags_async (
+    FolksTpLowlevel *tp_lowlevel,
+    TpConnection *conn,
+    GAsyncReadyCallback callback,
+    gpointer user_data)
+{
+  GSimpleAsyncResult *result;
+
+  result = g_simple_async_result_new (G_OBJECT (conn), callback, user_data,
+      folks_tp_lowlevel_connection_get_alias_flags_finish);
+
+  tp_cli_connection_interface_aliasing_call_get_alias_flags (conn, -1,
+      connection_get_alias_flags_cb, result, NULL, G_OBJECT (conn));
+}
+
+/**
+ * folks_tp_lowlevel_connection_get_alias_flags_finish:
+ * @lowlevel: a #FolksTpLowlevel
+ * @result: a #GAsyncResult
+ * @error: return location for a #GError, or %NULL
+ *
+ * Determine the alias-related capabilities of the #TpConnection.
+ *
+ * Returns: the #TpConnectionAliasFlags
+ */
+TpConnectionAliasFlags
+folks_tp_lowlevel_connection_get_alias_flags_finish (
+    FolksTpLowlevel *lowlevel,
+    GAsyncResult *result,
+    GError **error)
+{
+  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+  TpConnection *conn;
+
+  g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (simple), FALSE);
+
+  conn = TP_CONNECTION (g_async_result_get_source_object (result));
+  g_return_val_if_fail (TP_IS_CONNECTION (conn), FALSE);
+
+  if (g_simple_async_result_propagate_error (simple, error))
+    return 0;
+
+  g_return_val_if_fail (g_simple_async_result_is_valid (result,
+      G_OBJECT (conn), folks_tp_lowlevel_connection_get_alias_flags_finish),
+      0);
+
+  return (TpConnectionAliasFlags) (g_simple_async_result_get_op_res_gpointer (
+      G_SIMPLE_ASYNC_RESULT (result)));
+}
+
 static void
 get_contacts_by_handle_cb (TpConnection *conn,
     guint n_contacts,
@@ -348,6 +425,88 @@ folks_tp_lowlevel_connection_get_contacts_by_id_finish (
       G_SIMPLE_ASYNC_RESULT (result));
 }
 
+static void
+connection_get_requestable_channel_classes_cb (TpProxy *conn,
+    const GValue *value,
+    const GError *error,
+    gpointer user_data,
+    GObject *weak_object)
+{
+  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (user_data);
+  GPtrArray *props;
+
+  if (error != NULL)
+    {
+      g_simple_async_result_set_from_error (simple, error);
+    }
+  else
+    {
+      props = (GPtrArray*) g_value_get_boxed (value);
+      g_simple_async_result_set_op_res_gpointer (simple,
+          g_ptr_array_ref (props), (GDestroyNotify) g_ptr_array_unref);
+    }
+
+  g_simple_async_result_complete (simple);
+  g_object_unref (simple);
+}
+
+void
+folks_tp_lowlevel_connection_get_requestable_channel_classes_async (
+    FolksTpLowlevel *tp_lowlevel,
+    TpConnection *conn,
+    GAsyncReadyCallback callback,
+    gpointer user_data)
+{
+  GSimpleAsyncResult *result;
+
+  result = g_simple_async_result_new (G_OBJECT (conn), callback, user_data,
+      folks_tp_lowlevel_connection_get_requestable_channel_classes_finish);
+
+  tp_cli_dbus_properties_call_get (conn, -1,
+      TP_IFACE_CONNECTION_INTERFACE_REQUESTS, "RequestableChannelClasses",
+      connection_get_requestable_channel_classes_cb, result, NULL,
+      G_OBJECT (conn));
+}
+
+/**
+ * folks_tp_lowlevel_connection_get_requestable_channel_classes_finish:
+ * @tp_lowlevel: a #FolksTpLowlevel
+ * @result: a #GAsyncResult
+ * @error: return location for a #GError, or %NULL
+ *
+ * Retrieve the #TpConnection's RequestableChannelClasses D-Bus property.
+ *
+ * Returns: (transfer full): the boxed property details. Unref with
+ * g_ptr_array_unref().
+ */
+GPtrArray *
+folks_tp_lowlevel_connection_get_requestable_channel_classes_finish (
+    FolksTpLowlevel *tp_lowlevel,
+    GAsyncResult *result,
+    GError **error)
+{
+  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+  GPtrArray *props;
+  TpConnection *conn;
+
+  g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (simple), NULL);
+
+  conn = TP_CONNECTION (g_async_result_get_source_object (result));
+  g_return_val_if_fail (TP_IS_CONNECTION (conn), NULL);
+
+  if (g_simple_async_result_propagate_error (simple, error))
+    return NULL;
+
+  g_return_val_if_fail (g_simple_async_result_is_valid (result,
+      G_OBJECT (conn),
+      folks_tp_lowlevel_connection_get_requestable_channel_classes_finish),
+      NULL);
+
+  props = (GPtrArray*) g_simple_async_result_get_op_res_gpointer (
+      G_SIMPLE_ASYNC_RESULT (result));
+  return g_ptr_array_ref (props);
+}
+
 static void
 group_request_channel_cb (
     TpConnection *conn,
@@ -360,7 +519,8 @@ group_request_channel_cb (
    * handle the error if RequestChannel failed */
   if (error)
     {
-      g_message ("Error: %s", error->message);
+      /* Translators: the parameter is an error message. */
+      g_message (_("Error requesting a group channel: %s"), error->message);
       return;
     }
 }
@@ -377,7 +537,8 @@ group_request_handles_cb (
 
   if (error)
     {
-      g_message ("Error: %s", error->message);
+      /* Translators: the parameter is an error message. */
+      g_message (_("Error requesting group handles: %s"), error->message);
       return;
     }
 
@@ -415,7 +576,8 @@ set_contact_alias_cb (TpConnection *conn,
 {
   if (error != NULL)
     {
-      g_message ("Failed to change contact's alias: %s", error->message);
+      /* Translators: the parameter is an error message. */
+      g_message (_("Failed to change contact's alias: %s"), error->message);
       return;
     }
 }
@@ -474,7 +636,8 @@ iterate_on_channels (TpConnection *conn,
 
     channel = tp_channel_new_from_properties (conn, path, properties, &error);
     if (channel == NULL) {
-      g_message ("Failed to create group channel: %s", error->message);
+      /* Translators: the parameter is an error message. */
+      g_message (_("Failed to create group channel: %s"), error->message);
       g_error_free (error);
       return;
     }
@@ -503,7 +666,8 @@ got_channels_cb (TpProxy *conn,
   const GPtrArray *channels;
 
   if (error != NULL) {
-    g_message ("Get Channels property failed: %s", error->message);
+    /* Translators: the parameter is an error message. */
+    g_message (_("Get Channels property failed: %s"), error->message);
     return;
   }
 
@@ -545,7 +709,9 @@ group_add_members_cb (TpChannel *proxy,
 {
   if (error != NULL)
     {
-      g_message ("Failed to add contact to group %s: %s",
+      /* Translators: the first parameter is a group channel identifier and the
+       * second is an error message. */
+      g_message (_("Failed to add contact to group '%s': %s"),
           tp_channel_get_identifier (TP_CHANNEL (proxy)), error->message);
       return;
     }
@@ -559,7 +725,9 @@ group_remove_members_cb (TpChannel *proxy,
 {
   if (error != NULL)
     {
-      g_message ("Failed to remove contact from group %s: %s",
+      /* Translators: the first parameter is a group channel identifier and the
+       * second is an error message. */
+      g_message (_("Failed to remove contact from group '%s': %s"),
           tp_channel_get_identifier (TP_CHANNEL (proxy)), error->message);
       return;
     }
@@ -579,7 +747,9 @@ folks_tp_lowlevel_channel_group_change_membership (TpChannel *channel,
     {
       g_set_error (error, FOLKS_TP_LOWLEVEL_ERROR,
           FOLKS_TP_LOWLEVEL_ERROR_INVALID_ARGUMENT,
-          "invalid group channel %p to add handle %d to", channel, handle);
+          /* Translators: the first parameter is a pointer address and the
+           * second is a contact handle (numeric identifier). */
+          _("Invalid group channel %p to add handle %d to."), channel, handle);
     }
 
   handles = g_array_new (FALSE, TRUE, sizeof (guint));
